@@ -15,30 +15,15 @@
  *
  *   You should have received a copy of the GNU Lesser General Public
  *   License along with this library; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+ *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
-#include "Python.h"
-#include "structmember.h"
-#include "frameobject.h"
-#ifndef PY_LONG_LONG
-  #define PY_LONG_LONG LONG_LONG
-#endif
+#include "common.h"
 #include "sys/poll.h"
 #include "stdlib.h"
 #include "alsa/asoundlib.h"
 
-#ifndef Py_RETURN_NONE
-#define Py_RETURN_NONE return Py_INCREF(Py_None), Py_None
-#endif
-#ifndef Py_RETURN_TRUE
-#define Py_RETURN_TRUE return Py_INCREF(Py_True), Py_True
-#endif
-#ifndef Py_RETURN_FALSE
-#define Py_RETURN_FALSE return Py_INCREF(Py_False), Py_False
-#endif
-
 static int element_callback(snd_hctl_elem_t *elem, unsigned int mask);
 
 static PyObject *module;
@@ -59,17 +44,6 @@ struct pyalsahcontrol {
 	snd_hctl_t *handle;
 };
 
-static inline PyObject *get_bool(int val)
-{
-	if (val) {
-		Py_INCREF(Py_True);
-		return Py_True;
-	} else {
-		Py_INCREF(Py_False);
-		return Py_False;
-	}
-}
-
 static PyObject *id_to_python(snd_ctl_elem_id_t *id)
 {
 	PyObject *v;
@@ -81,7 +55,7 @@ static PyObject *id_to_python(snd_ctl_elem_id_t *id)
 	PyTuple_SET_ITEM(v, 1, PyInt_FromLong(snd_ctl_elem_id_get_interface(id)));
 	PyTuple_SET_ITEM(v, 2, PyInt_FromLong(snd_ctl_elem_id_get_device(id)));
 	PyTuple_SET_ITEM(v, 3, PyInt_FromLong(snd_ctl_elem_id_get_subdevice(id)));
-	PyTuple_SET_ITEM(v, 4, PyString_FromString(snd_ctl_elem_id_get_name(id)));
+	PyTuple_SET_ITEM(v, 4, PyUnicode_FromString(snd_ctl_elem_id_get_name(id)));
 	PyTuple_SET_ITEM(v, 5, PyInt_FromLong(snd_ctl_elem_id_get_index(id)));
 	return v;
 }
@@ -143,12 +117,16 @@ pyalsahcontrol_getChctl(struct pyalsahcontrol *self, void *priv)
 }
 
 PyDoc_STRVAR(handlevents__doc__,
-"handleEvents() -- Process waiting hcontrol events (and call appropriate callbacks).");
+"handle_events() -- Process waiting hcontrol events (and call appropriate callbacks).");
 
 static PyObject *
 pyalsahcontrol_handleevents(struct pyalsahcontrol *self, PyObject *args)
 {
-	int err = snd_hctl_handle_events(self->handle);
+	int err;
+	
+	Py_BEGIN_ALLOW_THREADS;
+	err = snd_hctl_handle_events(self->handle);
+	Py_END_ALLOW_THREADS;
 	if (err < 0)
 		PyErr_Format(PyExc_IOError,
 		     "HControl handle events error: %s", strerror(-err));
@@ -156,7 +134,7 @@ pyalsahcontrol_handleevents(struct pyalsahcontrol *self, PyObject *args)
 }
 
 PyDoc_STRVAR(registerpoll__doc__,
-"registerPoll(pollObj) -- Register poll file descriptors.");
+"register_poll(pollObj) -- Register poll file descriptors.");
 
 static PyObject *
 pyalsahcontrol_registerpoll(struct pyalsahcontrol *self, PyObject *args)
@@ -171,14 +149,12 @@ pyalsahcontrol_registerpoll(struct pyalsahcontrol *self, PyObject *args)
 	count = snd_hctl_poll_descriptors_count(self->handle);
 	if (count <= 0)
 		Py_RETURN_NONE;
-	pfd = malloc(sizeof(struct pollfd) * count);
-	if (pfd == NULL)
-		Py_RETURN_NONE;
+	pfd = alloca(sizeof(struct pollfd) * count);
 	count = snd_hctl_poll_descriptors(self->handle, pfd, count);
 	if (count <= 0)
 		Py_RETURN_NONE;
 	
-	reg = PyObject_GetAttr(pollObj, PyString_InternFromString("register"));
+	reg = PyObject_GetAttr(pollObj, InternFromString("register"));
 
 	for (i = 0; i < count; i++) {
 		t = PyTuple_New(2);
@@ -195,6 +171,38 @@ pyalsahcontrol_registerpoll(struct pyalsahcontrol *self, PyObject *args)
 	Py_RETURN_NONE;
 }
 
+static PyObject *
+pyalsahcontrol_getpollfds(struct pyalsahcontrol *self, void *priv)
+{
+	PyObject *l, *t;
+	struct pollfd *pfds;
+	int i, count;
+
+	count = snd_hctl_poll_descriptors_count(self->handle);
+	if (count < 0) {
+pfds_error:
+		PyErr_Format(PyExc_IOError, "poll descriptors error: %s", snd_strerror(count));
+		return NULL;
+	}
+	pfds = alloca(sizeof(struct pollfd) * count);
+	count = snd_hctl_poll_descriptors(self->handle, pfds, count);
+	if (count < 0)
+		goto pfds_error;
+
+	l = PyList_New(count);
+	if (!l)
+		return NULL;
+	for (i = 0; i < count; ++i) {
+		t = PyTuple_New(2);
+		if (t) {
+			PyTuple_SET_ITEM(t, 0, PyInt_FromLong(pfds[i].fd));
+			PyTuple_SET_ITEM(t, 1, PyInt_FromLong(pfds[i].events));
+			PyList_SetItem(l, i, t);
+		}
+	}
+	return l;
+}
+
 PyDoc_STRVAR(list__doc__,
 "list() -- Return a list (tuple) of element IDs in (numid,interface,device,subdevice,name,index) tuple.");
 
@@ -229,17 +237,17 @@ pyalsahcontrol_list(struct pyalsahcontrol *self, PyObject *args)
 }
 
 PyDoc_STRVAR(elementnew__doc__,
-"elementNew(elementType['Integer'], id, count, min, max, step)\n"
-"elementNew(elementType['Integer64'], id, count, min64, max64, step64)\n"
-"elementNew(elementType['Boolean'], id, count)\n"
-"elementNew(elementType['IEC958'], id)\n"
+"element_new(element_type['INTEGER'], id, count, min, max, step)\n"
+"element_new(element_type['INTEGER64'], id, count, min64, max64, step64)\n"
+"element_new(element_type['BOOLEAN'], id, count)\n"
+"element_new(element_type['IEC958'], id)\n"
 "  -- Create a new hcontrol element.\n"
 "  -- The id argument is tuple (interface, device, subdevice, name, index).\n");
 
 static PyObject *
 pyalsahcontrol_elementnew(struct pyalsahcontrol *self, PyObject *args)
 {
-	snd_ctl_elem_type_t type;
+	long type;
 	PyObject *o;
 	unsigned int count;
 	long min, max, step;
@@ -254,12 +262,11 @@ pyalsahcontrol_elementnew(struct pyalsahcontrol *self, PyObject *args)
 		return NULL;
 	}
 	o = PyTuple_GetItem(args, 0);
-	if (!PyInt_Check(o)) {
+	if (get_long1(o, &type)) {
 		PyErr_SetString(PyExc_TypeError, "type argument is not integer");
 		return NULL;
 	}
 	Py_INCREF(o);
-	type = PyInt_AsLong(o);
 	o = PyTuple_GetItem(args, 1);
 	if (!PyTuple_Check(o)) {
 		PyErr_SetString(PyExc_TypeError, "id argument is not tuple");
@@ -283,7 +290,7 @@ pyalsahcontrol_elementnew(struct pyalsahcontrol *self, PyObject *args)
 			return NULL;
 		break;
 	default:
-		PyErr_Format(PyExc_TypeError, "type %i is not supported yet", type);
+		PyErr_Format(PyExc_TypeError, "type %li is not supported yet", type);
 		return NULL;
 		
 	}
@@ -308,15 +315,15 @@ pyalsahcontrol_elementnew(struct pyalsahcontrol *self, PyObject *args)
 		break;
 	}
 	if (res < 0) {
-		PyErr_Format(PyExc_IOError, "new element of type %i create error: %s", type, snd_strerror(-res));
+		PyErr_Format(PyExc_IOError, "new element of type %li create error: %s", type, snd_strerror(-res));
 		return NULL;
 	}
 	Py_RETURN_NONE;
 }
 
 PyDoc_STRVAR(elementremove__doc__,
-"elementRemove(id)\n"
-"elementRemove(interface, device, subdevice, name, index)\n"
+"element_remove(id)\n"
+"element_remove(interface, device, subdevice, name, index)\n"
 "  -- Remove element.\n");
 
 static PyObject *
@@ -326,8 +333,8 @@ pyalsahcontrol_elementremove(struct pyalsahcontrol *self, PyObject *args)
 }
 
 PyDoc_STRVAR(elementlock__doc__,
-"elementLock(id)\n"
-"elementLock(interface, device, subdevice, name, index)\n"
+"element_lock(id)\n"
+"element_lock(interface, device, subdevice, name, index)\n"
 " -- Lock element.\n");
 
 static PyObject *
@@ -337,8 +344,8 @@ pyalsahcontrol_elementlock(struct pyalsahcontrol *self, PyObject *args)
 }
 
 PyDoc_STRVAR(elementunlock__doc__,
-"elementUnlock(id)\n"
-"elementUnlock(interface, device, subdevice, name, index)\n"
+"element_unlock(id)\n"
+"element_unlock(interface, device, subdevice, name, index)\n"
 " -- Unlock element.\n");
 
 static PyObject *
@@ -390,13 +397,13 @@ pyalsahcontrol_dealloc(struct pyalsahcontrol *self)
 {
 	if (self->handle != NULL)
 		snd_hctl_close(self->handle);
-
-	self->ob_type->tp_free(self);
+	Py_TYPE(self)->tp_free((PyObject*)self);
 }
 
 static PyGetSetDef pyalsahcontrol_getseters[] = {
 
 	{"count",	(getter)pyalsahcontrol_getcount,	NULL,	"hcontrol element count",		NULL},
+	{"poll_fds",	(getter)pyalsahcontrol_getpollfds,	NULL,	"list of (fd, eventbits) tuples",	NULL},
 	{"get_C_hctl",	(getter)pyalsahcontrol_getChctl,	NULL,	"hcontrol C pointer (internal)",	NULL},
 
 	{NULL}
@@ -405,17 +412,17 @@ static PyGetSetDef pyalsahcontrol_getseters[] = {
 static PyMethodDef pyalsahcontrol_methods[] = {
 
 	{"list",	(PyCFunction)pyalsahcontrol_list,	METH_NOARGS,	list__doc__},
-	{"elementNew",	(PyCFunction)pyalsahcontrol_elementnew,	METH_VARARGS,	elementnew__doc__},
-	{"elementRemove",(PyCFunction)pyalsahcontrol_elementremove,	METH_VARARGS,	elementremove__doc__},
-	{"elementLock",(PyCFunction)pyalsahcontrol_elementlock,	METH_VARARGS,	elementlock__doc__},
-	{"elementUnlock",(PyCFunction)pyalsahcontrol_elementunlock,	METH_VARARGS,	elementunlock__doc__},
-	{"handleEvents",(PyCFunction)pyalsahcontrol_handleevents,	METH_NOARGS,	handlevents__doc__},
-	{"registerPoll",(PyCFunction)pyalsahcontrol_registerpoll,	METH_VARARGS|METH_KEYWORDS,	registerpoll__doc__},
+	{"element_new",	(PyCFunction)pyalsahcontrol_elementnew,	METH_VARARGS,	elementnew__doc__},
+	{"element_remove",(PyCFunction)pyalsahcontrol_elementremove,	METH_VARARGS,	elementremove__doc__},
+	{"element_lock",(PyCFunction)pyalsahcontrol_elementlock,	METH_VARARGS,	elementlock__doc__},
+	{"element_unlock",(PyCFunction)pyalsahcontrol_elementunlock,	METH_VARARGS,	elementunlock__doc__},
+	{"handle_events",(PyCFunction)pyalsahcontrol_handleevents,	METH_NOARGS,	handlevents__doc__},
+	{"register_poll",(PyCFunction)pyalsahcontrol_registerpoll,	METH_VARARGS|METH_KEYWORDS,	registerpoll__doc__},
 	{NULL}
 };
 
 static PyTypeObject pyalsahcontrol_type = {
-	PyObject_HEAD_INIT(0)
+	PyVarObject_HEAD_INIT(NULL, 0)
 	tp_name:	"alsahcontrol.HControl",
 	tp_basicsize:	sizeof(struct pyalsahcontrol),
 	tp_dealloc:	(destructor)pyalsahcontrol_dealloc,
@@ -447,7 +454,7 @@ struct pyalsahcontrolelement {
 static PyObject *
 pyalsahcontrolelement_getname(struct pyalsahcontrolelement *pyhelem, void *priv)
 {
-	return PyString_FromString(snd_hctl_elem_get_name(pyhelem->elem));
+	return PyUnicode_FromString(snd_hctl_elem_get_name(pyhelem->elem));
 }
 
 typedef unsigned int (*fcn1)(void *);
@@ -499,7 +506,7 @@ pyalsahcontrolelement_unlock(struct pyalsahcontrolelement *pyhelem, PyObject *ar
 }
 
 PyDoc_STRVAR(setcallback__doc__,
-"setCallback(callObj) -- Set callback object.\n"
+"set_callback(callObj) -- Set callback object.\n"
 "Note: callObj might have callObj.callback attribute.\n");
 
 static PyObject *
@@ -535,9 +542,10 @@ pyalsahcontrolelement_init(struct pyalsahcontrolelement *pyhelem, PyObject *args
 	char *name = "Default";
 	int numid = 0, iface = 0, device = 0, subdevice = 0, index = 0;
 	snd_ctl_elem_id_t *id;
-	static char *kwlist1[] = { "hctl", "interface", "device", "subdevice", "name", "index" };
+	static char *kwlist1[] = { "hctl", "interface", "device", "subdevice", "name", "index", NULL };
 	long helem = 0;
 	float f = 0;
+	snd_ctl_elem_info_t *elem_info;
 
 	snd_ctl_elem_id_alloca(&id);
 	pyhelem->pyhandle = NULL;
@@ -552,14 +560,20 @@ pyalsahcontrolelement_init(struct pyalsahcontrolelement *pyhelem, PyObject *args
 	if (PyFloat_Check(first)) {
 		if (!PyArg_ParseTuple(args, "Ofi", &hctl, &f, &helem))
 			return -1;
+	} else if (PyLong_Check(first)) {
+		if (!PyArg_ParseTuple(args, "Oi", &hctl, &numid))
+			return -1;
+		snd_ctl_elem_id_set_numid(id, numid);
+#if PY_MAJOR_VERSION < 3
 	} else if (PyInt_Check(first)) {
 		if (!PyArg_ParseTuple(args, "Oi", &hctl, &numid))
 			return -1;
 		snd_ctl_elem_id_set_numid(id, numid);
+#endif
 	} else if (PyTuple_Check(first)) {
 		if (!PyArg_ParseTuple(args, "OO", &hctl, &first))
 			return -1;
-		if (!PyArg_ParseTupleAndKeywords(first, kwds, "|iiisi", kwlist1 + 1, &iface, &device, &subdevice, &name, &index))
+		if (!PyArg_ParseTuple(first, "iiis|i", &iface, &device, &subdevice, &name, &index))
 			return -1;
 		goto parse1;
 	} else {
@@ -582,9 +596,17 @@ pyalsahcontrolelement_init(struct pyalsahcontrolelement *pyhelem, PyObject *args
 	Py_INCREF(hctl);
 	pyhelem->handle = PYHCTL(hctl)->handle;
 
+	/* FIXME: Remove it when hctl find works ok !!! */
+	snd_ctl_elem_info_alloca(&elem_info);
+	snd_ctl_elem_info_set_id(elem_info, id);
+	if (snd_ctl_elem_info(snd_hctl_ctl(pyhelem->handle), elem_info) < 0)
+		goto elem_not_found;
+	snd_ctl_elem_info_get_id(elem_info, id);
+
 	pyhelem->elem = helem > 0 ? (snd_hctl_elem_t *)helem :
 				snd_hctl_find_elem(pyhelem->handle, id);
 	if (pyhelem->elem == NULL) {
+elem_not_found:
 		if (numid == 0)
 			PyErr_Format(PyExc_IOError, "cannot find hcontrol element %i,%i,%i,'%s',%i", iface, device, subdevice, name, index);
 		else
@@ -599,14 +621,11 @@ static void
 pyalsahcontrolelement_dealloc(struct pyalsahcontrolelement *self)
 {
 	if (self->elem) {
-		Py_XDECREF(self->callback);
+		Py_DECREF(self->callback);
 		snd_hctl_elem_set_callback(self->elem, NULL);
 	}
-	if (self->pyhandle) {
-		Py_XDECREF(self->pyhandle);
-	}
-
-	self->ob_type->tp_free(self);
+	Py_XDECREF(self->pyhandle);
+	Py_TYPE(self)->tp_free((PyObject*)self);
 }
 
 static PyGetSetDef pyalsahcontrolelement_getseters[] = {
@@ -627,13 +646,13 @@ static PyMethodDef pyalsahcontrolelement_methods[] = {
 	{"lock",	(PyCFunction)pyalsahcontrolelement_lock,	METH_NOARGS,	elock__doc__},
 	{"unlock",	(PyCFunction)pyalsahcontrolelement_unlock,	METH_NOARGS,	eunlock__doc__},
 
-	{"setCallback",	(PyCFunction)pyalsahcontrolelement_setcallback,	METH_VARARGS,	setcallback__doc__},
+	{"set_callback",(PyCFunction)pyalsahcontrolelement_setcallback,	METH_VARARGS,	setcallback__doc__},
 
 	{NULL}
 };
 
 static PyTypeObject pyalsahcontrolelement_type = {
-	PyObject_HEAD_INIT(0)
+	PyVarObject_HEAD_INIT(NULL, 0)
 	tp_name:	"alsahcontrol.Element",
 	tp_basicsize:	sizeof(struct pyalsahcontrolelement),
 	tp_dealloc:	(destructor)pyalsahcontrolelement_dealloc,
@@ -729,7 +748,7 @@ typedef const char * (*fcn5)(void *);
 static PyObject *
 pyalsahcontrolinfo_str(struct pyalsahcontrolinfo *pyinfo, void *fcn)
 {
-	return PyString_FromString(((fcn5)fcn)(pyinfo->info));
+	return PyUnicode_FromString(((fcn5)fcn)(pyinfo->info));
 }
 
 static PyObject *
@@ -746,7 +765,7 @@ static PyObject *
 pyalsahcontrolinfo_dimensions(struct pyalsahcontrolinfo *pyinfo, void *priv)
 {
 	int dims = snd_ctl_elem_info_get_dimensions(pyinfo->info);
-	unsigned int i;
+	int i;
 	PyObject *t;
 	
 	if (dims <= 0)
@@ -765,7 +784,7 @@ pyalsahcontrolinfo_itemnames(struct pyalsahcontrolinfo *pyinfo, void *priv)
 {
 	int items;
 	int res;
-	unsigned int i;
+	int i;
 	PyObject *t;
 	
 	if (snd_ctl_elem_info_get_type(pyinfo->info) != SND_CTL_ELEM_TYPE_ENUMERATED) {
@@ -785,7 +804,7 @@ pyalsahcontrolinfo_itemnames(struct pyalsahcontrolinfo *pyinfo, void *priv)
 			Py_INCREF(Py_None);
 			PyTuple_SET_ITEM(t, i, Py_None);
 		} else {
-			PyTuple_SET_ITEM(t, i, PyString_FromString(snd_ctl_elem_info_get_item_name(pyinfo->info)));
+			PyTuple_SET_ITEM(t, i, PyUnicode_FromString(snd_ctl_elem_info_get_item_name(pyinfo->info)));
 		}
 	}
 	return t;
@@ -836,11 +855,8 @@ pyalsahcontrolinfo_dealloc(struct pyalsahcontrolinfo *self)
 {
 	if (self->info)
 		snd_ctl_elem_info_free(self->info);
-	if (self->pyelem) {
-		Py_XDECREF(self->pyelem);
-	}
-
-	self->ob_type->tp_free(self);
+	Py_XDECREF(self->pyelem);
+	Py_TYPE(self)->tp_free((PyObject*)self);
 }
 
 static PyGetSetDef pyalsahcontrolinfo_getseters[] = {
@@ -858,16 +874,16 @@ static PyGetSetDef pyalsahcontrolinfo_getseters[] = {
 
 	{"type",	(getter)pyalsahcontrolinfo_uint,	NULL,	"hcontrol element index",	snd_ctl_elem_info_get_type},
 
-	{"isReadable",	(getter)pyalsahcontrolinfo_bool,	NULL,	"hcontrol element is readable",	snd_ctl_elem_info_is_readable},
-	{"isWritable",	(getter)pyalsahcontrolinfo_bool,	NULL,	"hcontrol element is writable",snd_ctl_elem_info_is_writable},
-	{"isVolatile",	(getter)pyalsahcontrolinfo_bool,	NULL,	"hcontrol element is volatile",snd_ctl_elem_info_is_volatile},
-	{"isInactive",	(getter)pyalsahcontrolinfo_bool,	NULL,	"hcontrol element is inactive",snd_ctl_elem_info_is_inactive},
-	{"isLocked",	(getter)pyalsahcontrolinfo_bool,	NULL,	"hcontrol element is locked",snd_ctl_elem_info_is_locked},
-	{"isTlvReadable",(getter)pyalsahcontrolinfo_bool,	NULL,	"hcontrol element is TLV readable",snd_ctl_elem_info_is_tlv_readable},
-	{"isTlvWritable",(getter)pyalsahcontrolinfo_bool,	NULL,	"hcontrol element is TLV writable",snd_ctl_elem_info_is_tlv_writable},
-	{"isTlvCommandable",(getter)pyalsahcontrolinfo_bool,	NULL,	"hcontrol element is TLV commandable",snd_ctl_elem_info_is_tlv_commandable},
-	{"isOwner",	(getter)pyalsahcontrolinfo_bool,	NULL,	"this process is owner of this hcontrol element",snd_ctl_elem_info_is_owner},
-	{"isUser",	(getter)pyalsahcontrolinfo_bool,	NULL,	"hcontrol element is user element",snd_ctl_elem_info_is_user},
+	{"is_readable",	(getter)pyalsahcontrolinfo_bool,	NULL,	"hcontrol element is readable",	snd_ctl_elem_info_is_readable},
+	{"is_writable",	(getter)pyalsahcontrolinfo_bool,	NULL,	"hcontrol element is writable",snd_ctl_elem_info_is_writable},
+	{"is_volatile",	(getter)pyalsahcontrolinfo_bool,	NULL,	"hcontrol element is volatile",snd_ctl_elem_info_is_volatile},
+	{"is_inactive",	(getter)pyalsahcontrolinfo_bool,	NULL,	"hcontrol element is inactive",snd_ctl_elem_info_is_inactive},
+	{"is_locked",	(getter)pyalsahcontrolinfo_bool,	NULL,	"hcontrol element is locked",snd_ctl_elem_info_is_locked},
+	{"is_tlv_readable",(getter)pyalsahcontrolinfo_bool,	NULL,	"hcontrol element is TLV readable",snd_ctl_elem_info_is_tlv_readable},
+	{"is_tlv_Writable",(getter)pyalsahcontrolinfo_bool,	NULL,	"hcontrol element is TLV writable",snd_ctl_elem_info_is_tlv_writable},
+	{"is_tlv_Commandable",(getter)pyalsahcontrolinfo_bool,	NULL,	"hcontrol element is TLV commandable",snd_ctl_elem_info_is_tlv_commandable},
+	{"is_owner",	(getter)pyalsahcontrolinfo_bool,	NULL,	"this process is owner of this hcontrol element",snd_ctl_elem_info_is_owner},
+	{"is_user",	(getter)pyalsahcontrolinfo_bool,	NULL,	"hcontrol element is user element",snd_ctl_elem_info_is_user},
 
 	{"owner",	(getter)pyalsahcontrolinfo_getowner,	NULL,	"get owner pid for this hcontrol element",	NULL},
 	{"count",	(getter)pyalsahcontrolinfo_uint,	NULL,	"get count of values",			snd_ctl_elem_info_get_count},
@@ -880,7 +896,7 @@ static PyGetSetDef pyalsahcontrolinfo_getseters[] = {
 	{"items",	(getter)pyalsahcontrolinfo_getitems,	NULL,	"get count of enumerated items",	NULL},
 
 	{"dimensions",	(getter)pyalsahcontrolinfo_dimensions,	NULL,	"get hcontrol element dimensions (in tuple)",	NULL},
-	{"itemNames",	(getter)pyalsahcontrolinfo_itemnames,	NULL,	"get enumerated item names (in tuple)",		NULL},
+	{"item_names",	(getter)pyalsahcontrolinfo_itemnames,	NULL,	"get enumerated item names (in tuple)",		NULL},
 	
 	{NULL}
 };
@@ -890,7 +906,7 @@ static PyMethodDef pyalsahcontrolinfo_methods[] = {
 };
 
 static PyTypeObject pyalsahcontrolinfo_type = {
-	PyObject_HEAD_INIT(0)
+	PyVarObject_HEAD_INIT(NULL, 0)
 	tp_name:	"alsahcontrol.Info",
 	tp_basicsize:	sizeof(struct pyalsahcontrolinfo),
 	tp_dealloc:	(destructor)pyalsahcontrolinfo_dealloc,
@@ -938,7 +954,7 @@ typedef const char * (*fcn11)(void *);
 static PyObject *
 pyalsahcontrolvalue_str(struct pyalsahcontrolvalue *pyvalue, void *fcn)
 {
-	return PyString_FromString(((fcn5)fcn)(pyvalue->value));
+	return PyUnicode_FromString(((fcn5)fcn)(pyvalue->value));
 }
 
 static PyObject *
@@ -952,10 +968,10 @@ pyalsahcontrolvalue_id(struct pyalsahcontrolvalue *pyvalue, void *priv)
 }
 
 PyDoc_STRVAR(gettuple__doc__,
-"getTuple(type, count) -- Get hcontrol element values in tuple.");
+"get_tuple(type, count) -- Get hcontrol element values in tuple.");
 
 PyDoc_STRVAR(getarray__doc__,
-"getArray(type, count) -- Get hcontrol element values in array.");
+"get_array(type, count) -- Get hcontrol element values in array.");
 
 static PyObject *
 pyalsahcontrolvalue_get1(struct pyalsahcontrolvalue *self, PyObject *args, int list)
@@ -1036,13 +1052,13 @@ pyalsahcontrolvalue_get1(struct pyalsahcontrolvalue *self, PyObject *args, int l
 		}
 		snd_ctl_elem_value_get_iec958(self->value, iec958);
 		if (!list) {
-			PyTuple_SET_ITEM(t, 0, PyString_FromStringAndSize(iec958->status, sizeof(iec958->status)));
-			PyTuple_SET_ITEM(t, 1, PyString_FromStringAndSize(iec958->subcode, sizeof(iec958->subcode)));
-			PyTuple_SET_ITEM(t, 2, PyString_FromStringAndSize(iec958->dig_subframe, sizeof(iec958->dig_subframe)));
+			PyTuple_SET_ITEM(t, 0, PyUnicode_FromStringAndSize((char *)iec958->status, sizeof(iec958->status)));
+			PyTuple_SET_ITEM(t, 1, PyUnicode_FromStringAndSize((char *)iec958->subcode, sizeof(iec958->subcode)));
+			PyTuple_SET_ITEM(t, 2, PyUnicode_FromStringAndSize((char *)iec958->dig_subframe, sizeof(iec958->dig_subframe)));
 		} else {
-			PyList_SetItem(t, 0, PyString_FromStringAndSize(iec958->status, sizeof(iec958->status)));
-			PyList_SetItem(t, 1, PyString_FromStringAndSize(iec958->subcode, sizeof(iec958->subcode)));
-			PyList_SetItem(t, 2, PyString_FromStringAndSize(iec958->dig_subframe, sizeof(iec958->dig_subframe)));
+			PyList_SetItem(t, 0, PyUnicode_FromStringAndSize((char *)iec958->status, sizeof(iec958->status)));
+			PyList_SetItem(t, 1, PyUnicode_FromStringAndSize((char *)iec958->subcode, sizeof(iec958->subcode)));
+			PyList_SetItem(t, 2, PyUnicode_FromStringAndSize((char *)iec958->dig_subframe, sizeof(iec958->dig_subframe)));
 		}
 		free(iec958);
 		break;
@@ -1070,16 +1086,17 @@ pyalsahcontrolvalue_getarray(struct pyalsahcontrolvalue *self, PyObject *args)
 
 
 PyDoc_STRVAR(settuple__doc__,
-"setTuple(type, val) -- Set hcontrol element values from tuple.");
+"set_tuple(type, val) -- Set hcontrol element values from tuple.");
 
 PyDoc_STRVAR(setarray__doc__,
-"setArray(type, val) -- Set hcontrol element values from array.");
+"set_array(type, val) -- Set hcontrol element values from array.");
 
 static PyObject *
 pyalsahcontrolvalue_settuple(struct pyalsahcontrolvalue *self, PyObject *args)
 {
-	int type, len, list;
-	long i, count;
+	int type, list;
+	Py_ssize_t len;
+	long i, count, lval;
 	snd_aes_iec958_t *iec958;
 	PyObject *t, *v;
 	char *str;
@@ -1102,7 +1119,9 @@ pyalsahcontrolvalue_settuple(struct pyalsahcontrolvalue *self, PyObject *args)
 			if (v == Py_None)
 				continue;
 			Py_INCREF(v);
-			snd_ctl_elem_value_set_boolean(self->value, i, PyInt_AsLong(v));
+			if (get_long(v, &lval))
+				break;
+			snd_ctl_elem_value_set_boolean(self->value, i, lval);
 		}
 		break;
 	case SND_CTL_ELEM_TYPE_INTEGER:
@@ -1111,7 +1130,9 @@ pyalsahcontrolvalue_settuple(struct pyalsahcontrolvalue *self, PyObject *args)
 			if (v == Py_None)
 				continue;
 			Py_INCREF(v);
-			snd_ctl_elem_value_set_integer(self->value, i, PyInt_AsLong(v));
+			if (get_long(v, &lval))
+				break;
+			snd_ctl_elem_value_set_integer(self->value, i, lval);
 		}
 		break;
 	case SND_CTL_ELEM_TYPE_INTEGER64:
@@ -1129,7 +1150,9 @@ pyalsahcontrolvalue_settuple(struct pyalsahcontrolvalue *self, PyObject *args)
 			if (v == Py_None)
 				continue;
 			Py_INCREF(v);
-			snd_ctl_elem_value_set_enumerated(self->value, i, PyInt_AsLong(v));
+			if (get_long(v, &lval))
+				break;
+			snd_ctl_elem_value_set_enumerated(self->value, i, lval);
 		}
 		break;
 	case SND_CTL_ELEM_TYPE_BYTES:
@@ -1138,7 +1161,9 @@ pyalsahcontrolvalue_settuple(struct pyalsahcontrolvalue *self, PyObject *args)
 			if (v == Py_None)
 				continue;
 			Py_INCREF(v);
-			snd_ctl_elem_value_set_byte(self->value, i, PyInt_AsLong(v));
+			if (get_long(v, &lval))
+				break;
+			snd_ctl_elem_value_set_byte(self->value, i, lval);
 		}
 		break;
 	case SND_CTL_ELEM_TYPE_IEC958:
@@ -1154,25 +1179,25 @@ pyalsahcontrolvalue_settuple(struct pyalsahcontrolvalue *self, PyObject *args)
 		len = 0;
 		v = !list ? PyTuple_GET_ITEM(t, 0) : PyList_GetItem(t, 0);
 		Py_INCREF(v);
-		if (PyString_AsStringAndSize(v, &str, &len))
+		if (PyBytes_AsStringAndSize(v, &str, &len))
 			goto err1;
-		if (len > sizeof(iec958->status))
+		if (len > (Py_ssize_t)sizeof(iec958->status))
 			len = sizeof(iec958->status);
 		memcpy(iec958->status, str, len);
 		len = 0;
 		v = !list ? PyTuple_GET_ITEM(t, 1) : PyList_GetItem(t, 1);
 		Py_INCREF(v);
-		if (PyString_AsStringAndSize(v, &str, &len))
+		if (PyBytes_AsStringAndSize(v, &str, &len))
 			goto err1;
-		if (len > sizeof(iec958->subcode))
+		if (len > (Py_ssize_t)sizeof(iec958->subcode))
 			len = sizeof(iec958->subcode);
 		memcpy(iec958->subcode, str, len);
 		len = 0;
 		v = !list ? PyTuple_GET_ITEM(t, 2) : PyList_GetItem(t, 2);
 		Py_INCREF(v);
-		if (PyString_AsStringAndSize(v, &str, &len))
+		if (PyBytes_AsStringAndSize(v, &str, &len))
 			goto err1;
-		if (len > sizeof(iec958->dig_subframe))
+		if (len > (Py_ssize_t)sizeof(iec958->dig_subframe))
 			len = sizeof(iec958->dig_subframe);
 		memcpy(iec958->dig_subframe, str, len);
 		free(iec958);
@@ -1263,11 +1288,8 @@ pyalsahcontrolvalue_dealloc(struct pyalsahcontrolvalue *self)
 {
 	if (self->value)
 		snd_ctl_elem_value_free(self->value);
-	if (self->pyelem) {
-		Py_XDECREF(self->pyelem);
-	}
-
-	self->ob_type->tp_free(self);
+	Py_XDECREF(self->pyelem);
+	Py_TYPE(self)->tp_free((PyObject*)self);
 }
 
 static PyGetSetDef pyalsahcontrolvalue_getseters[] = {
@@ -1288,10 +1310,10 @@ static PyGetSetDef pyalsahcontrolvalue_getseters[] = {
 
 static PyMethodDef pyalsahcontrolvalue_methods[] = {
 
-	{"getTuple",	(PyCFunction)pyalsahcontrolvalue_gettuple,	METH_VARARGS,	gettuple__doc__},
-	{"getArray",	(PyCFunction)pyalsahcontrolvalue_getarray,	METH_VARARGS,	getarray__doc__},
-	{"setTuple",	(PyCFunction)pyalsahcontrolvalue_settuple,	METH_VARARGS,	settuple__doc__},
-	{"setArray",	(PyCFunction)pyalsahcontrolvalue_settuple,	METH_VARARGS,	setarray__doc__},
+	{"get_tuple",	(PyCFunction)pyalsahcontrolvalue_gettuple,	METH_VARARGS,	gettuple__doc__},
+	{"get_array",	(PyCFunction)pyalsahcontrolvalue_getarray,	METH_VARARGS,	getarray__doc__},
+	{"set_tuple",	(PyCFunction)pyalsahcontrolvalue_settuple,	METH_VARARGS,	settuple__doc__},
+	{"set_array",	(PyCFunction)pyalsahcontrolvalue_settuple,	METH_VARARGS,	setarray__doc__},
 
 	{"read",	(PyCFunction)pyalsahcontrolvalue_read,		METH_NOARGS,	read__doc__},
 	{"write",	(PyCFunction)pyalsahcontrolvalue_write,		METH_NOARGS,	write__doc__},
@@ -1300,7 +1322,7 @@ static PyMethodDef pyalsahcontrolvalue_methods[] = {
 };
 
 static PyTypeObject pyalsahcontrolvalue_type = {
-	PyObject_HEAD_INIT(0)
+	PyVarObject_HEAD_INIT(NULL, 0)
 	tp_name:	"alsahcontrol.Value",
 	tp_basicsize:	sizeof(struct pyalsahcontrolvalue),
 	tp_dealloc:	(destructor)pyalsahcontrolvalue_dealloc,
@@ -1322,31 +1344,30 @@ static PyMethodDef pyalsahcontrolparse_methods[] = {
 	{NULL}
 };
 
-PyMODINIT_FUNC
-initalsahcontrol(void)
+MOD_INIT(alsahcontrol)
 {
 	PyObject *d, *d1, *l1, *o;
 	int i;
 
 	if (PyType_Ready(&pyalsahcontrol_type) < 0)
-		return;
+		return MOD_ERROR_VAL;
 	if (PyType_Ready(&pyalsahcontrolelement_type) < 0)
-		return;
+		return MOD_ERROR_VAL;
 	if (PyType_Ready(&pyalsahcontrolinfo_type) < 0)
-		return;
+		return MOD_ERROR_VAL;
 	if (PyType_Ready(&pyalsahcontrolvalue_type) < 0)
-		return;
+		return MOD_ERROR_VAL;
 
-	module = Py_InitModule3("alsahcontrol", pyalsahcontrolparse_methods, "libasound hcontrol wrapper");
+	MOD_DEF(module, "alsahcontrol", "libasound hcontrol wrapper", pyalsahcontrolparse_methods);
 	if (module == NULL)
-		return;
+		return MOD_ERROR_VAL;
 
 #if 0
 	buildin = PyImport_AddModule("__buildin__");
 	if (buildin == NULL)
-		return;
+		return MOD_ERROR_VAL;
 	if (PyObject_SetAttrString(module, "__buildins__", buildin) < 0)
-		return;
+		return MOD_ERROR_VAL;
 #endif
 
 	Py_INCREF(&pyalsahcontrol_type);
@@ -1372,16 +1393,16 @@ initalsahcontrol(void)
 	PyDict_SetItemString(d1, pname, o); \
 	Py_DECREF(o); }
 	
-	add_space1("Card", CARD);
-	add_space1("HwDep", HWDEP);
-	add_space1("Mixer", MIXER);
+	add_space1("CARD", CARD);
+	add_space1("HWDEP", HWDEP);
+	add_space1("MIXER", MIXER);
 	add_space1("PCM", PCM);
-	add_space1("RawMidi", RAWMIDI);
-	add_space1("Timer", TIMER);
-	add_space1("Sequencer", SEQUENCER);
-	add_space1("Last", LAST);
+	add_space1("RAWMIDI", RAWMIDI);
+	add_space1("TIMER", TIMER);
+	add_space1("SEQUENCER", SEQUENCER);
+	add_space1("LAST", LAST);
 
-	PyDict_SetItemString(d, "InterfaceId", d1);
+	PyDict_SetItemString(d, "interface_id", d1);
 	Py_DECREF(d1);
 
 	/* ---- */
@@ -1389,12 +1410,12 @@ initalsahcontrol(void)
 	l1 = PyList_New(0);
 
 	for (i = 0; i <= SND_CTL_ELEM_IFACE_LAST; i++) {
-		o = PyString_FromString(snd_ctl_elem_iface_name(i));
+		o = PyUnicode_FromString(snd_ctl_elem_iface_name(i));
 		PyList_Append(l1, o);
 		Py_DECREF(o);
 	}
 
-	PyDict_SetItemString(d, "InterfaceName", l1);
+	PyDict_SetItemString(d, "interface_name", l1);
 	Py_DECREF(l1);
 
 	/* ---- */
@@ -1406,16 +1427,16 @@ initalsahcontrol(void)
 	PyDict_SetItemString(d1, pname, o); \
 	Py_DECREF(o); }
 	
-	add_space2("None", NONE);
-	add_space2("Boolean", BOOLEAN);
-	add_space2("Integer", INTEGER);
-	add_space2("Enumerated", ENUMERATED);
-	add_space2("Bytes", BYTES);
+	add_space2("NONE", NONE);
+	add_space2("BOOLEAN", BOOLEAN);
+	add_space2("INTEGER", INTEGER);
+	add_space2("ENUMERATED", ENUMERATED);
+	add_space2("BYTES", BYTES);
 	add_space2("IEC958", IEC958);
-	add_space2("Integer64", INTEGER64);
-	add_space2("Last", LAST);
+	add_space2("INTEGER64", INTEGER64);
+	add_space2("LAST", LAST);
 
-	PyDict_SetItemString(d, "ElementType", d1);
+	PyDict_SetItemString(d, "element_type", d1);
 	Py_DECREF(d1);
 	
 	/* ---- */
@@ -1423,12 +1444,12 @@ initalsahcontrol(void)
 	l1 = PyList_New(0);
 
 	for (i = 0; i <= SND_CTL_ELEM_TYPE_LAST; i++) {
-		o = PyString_FromString(snd_ctl_elem_type_name(i));
+		o = PyUnicode_FromString(snd_ctl_elem_type_name(i));
 		PyList_Append(l1, o);
 		Py_DECREF(o);
 	}
 
-	PyDict_SetItemString(d, "ElementTypeName", l1);
+	PyDict_SetItemString(d, "element_type_name", l1);
 	Py_DECREF(l1);
 
 	/* ---- */
@@ -1440,10 +1461,10 @@ initalsahcontrol(void)
 	PyDict_SetItemString(d1, pname, o); \
 	Py_DECREF(o); }
 	
-	add_space3("Element", ELEM);
-	add_space3("Last", LAST);
+	add_space3("ELEMENT", ELEM);
+	add_space3("LAST", LAST);
 
-	PyDict_SetItemString(d, "EventClass", d1);
+	PyDict_SetItemString(d, "event_class", d1);
 	Py_DECREF(d1);
 
 	/* ---- */
@@ -1455,16 +1476,16 @@ initalsahcontrol(void)
 	PyDict_SetItemString(d1, pname, o); \
 	Py_DECREF(o); }
 	
-	add_space4("Value", VALUE);
-	add_space4("Info", INFO);
-	add_space4("Add", ADD);
+	add_space4("VALUE", VALUE);
+	add_space4("INFO", INFO);
+	add_space4("ADD", ADD);
 	add_space4("TLV", TLV);
 
-	PyDict_SetItemString(d, "EventMask", d1);
+	PyDict_SetItemString(d, "event_mask", d1);
 	Py_DECREF(d1);
 
 	o = PyInt_FromLong(SND_CTL_EVENT_MASK_REMOVE);
-	PyDict_SetItemString(d, "EventMaskRemove", o);
+	PyDict_SetItemString(d, "event_mask_remove", o);
 	Py_DECREF(o);
 
 	/* ---- */
@@ -1476,11 +1497,11 @@ initalsahcontrol(void)
 	PyDict_SetItemString(d1, pname, o); \
 	Py_DECREF(o); }
 	
-	add_space5("NonBlock", NONBLOCK);
-	add_space5("Async", ASYNC);
-	add_space5("ReadOnly", READONLY);
+	add_space5("NONBLOCK", NONBLOCK);
+	add_space5("ASYNC", ASYNC);
+	add_space5("READONLY", READONLY);
 
-	PyDict_SetItemString(d, "OpenMode", d1);
+	PyDict_SetItemString(d, "open_mode", d1);
 	Py_DECREF(d1);
 
 	/* ---- */
@@ -1489,6 +1510,8 @@ initalsahcontrol(void)
 
 	if (PyErr_Occurred())
 		Py_FatalError("Cannot initialize module alsahcontrol");
+
+	return MOD_SUCCESS_VAL(module);
 }
 
 /*
@@ -1511,7 +1534,7 @@ static int element_callback(snd_hctl_elem_t *elem, unsigned int mask)
 	tstate = PyThreadState_New(main_interpreter);
 	origstate = PyThreadState_Swap(tstate);
 
-	o = PyObject_GetAttr(pyhelem->callback, PyString_InternFromString("callback"));
+	o = PyObject_GetAttr(pyhelem->callback, InternFromString("callback"));
 	if (!o) {
 		PyErr_Clear();
 		o = pyhelem->callback;
@@ -1520,15 +1543,19 @@ static int element_callback(snd_hctl_elem_t *elem, unsigned int mask)
 
 	t = PyTuple_New(2);
 	if (t) {
-		if (PyTuple_SET_ITEM(t, 0, (PyObject *)pyhelem))
-			Py_INCREF(pyhelem);
+		PyTuple_SET_ITEM(t, 0, (PyObject *)pyhelem);
+		Py_INCREF(pyhelem);
 		PyTuple_SET_ITEM(t, 1, PyInt_FromLong(mask));
 		r = PyObject_CallObject(o, t);
 		Py_DECREF(t);
 			
 		if (r) {
-			if (PyInt_Check(r)) {
+			if (PyLong_Check(r)) {
+				res = PyLong_AsLong(r);
+#if PY_MAJOR_VERSION < 3
+			} else if (PyInt_Check(r)) {
 				res = PyInt_AsLong(r);
+#endif
 			} else if (r == Py_None) {
 				res = 0;
 			}
